package edu.uoc.grp.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import javax.servlet.http.HttpSession;
import java.util.LinkedHashMap;
import java.util.Map;
import edu.uoc.grp.exceptions.ExcepcioUsuari;
import edu.uoc.grp.service.ServeiUsuari;
import edu.uoc.grp.model.Usuari;
import edu.uoc.grp.model.Empleat;
import edu.uoc.grp.utilities.Utils;


/**
 * Classe ControladorAdministrador.
 * 
 * Classe de tipus controlador encarregada de gestionar les peticions i les respostes relacionades amb les funcionalitats del administrador. 
 *
 * @version 1.0; java version "1.8.0";
 */
@Controller
@RequestMapping("/administrador") 
public class ControladorAdministrador {
	
	private static final Logger LOGGER = LogManager.getLogger(ControladorAdministrador.class.getName());
    
    @Autowired
    private ServeiUsuari serveiUsuari;

    @Autowired
    private Utils utils;
    
    
    @PostMapping("/altaUsuari")
    public String altaUsuari(Model model, @ModelAttribute("idEmpleat") Integer idEmpleat) {

		LOGGER.info("altaUsuari");
		Empleat empleat;
		
		LOGGER.debug("idEmpleat: " + idEmpleat);		
		
        try {

        	empleat = serveiUsuari.cercarEmpleat(idEmpleat);
		
        } catch (ExcepcioUsuari eu) {

        	String missatgeError = "Error general de bbdd de tipus: " + eu.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }

		model.addAttribute("empleat", empleat);

		return "altaUsuari";
    }


	@PostMapping("/guardarAltaUsuari")
    public String guardarAltaUsuari(HttpSession session, Model model, @ModelAttribute("idUsuari") Integer idUsuari, @ModelAttribute("tipusUsuari") String tipusUsuari, 
    		@ModelAttribute("login") String login, @ModelAttribute("contrasenya") String contrasenya, @ModelAttribute("eMail") String eMail) {

		LOGGER.info("guardarAltaUsuari");

		LOGGER.debug("idUsuari: " + idUsuari);
		LOGGER.debug("tipusUsuari: " + tipusUsuari);	
		
        //Recuperar el usuari de dins de la sessi.
		String usuariAdministrador = (String) session.getAttribute("usuari");

		Map<String, String> resultatMap = serveiUsuari.altaUsuari(usuariAdministrador, idUsuari, tipusUsuari, login, contrasenya, eMail);

		String resultat = (String) resultatMap.get("RESULTAT");		

        //En cas d'error, mostrem a l'usuari el missatge informatiu de l'error.
        if (resultat.equalsIgnoreCase("NOK")) {
        	
        	String missatgeError = (String) resultatMap.get("MISSATGE");
            LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }

        //Si l'alta d'usuari s correcta s'informa a l'usuari d'aquest fet.        
        else if (resultat.equalsIgnoreCase("OK")) {
        	
            String missatgeInfo = (String) resultatMap.get("MISSATGE");
            LOGGER.info("INFO: "+ missatgeInfo);            
            
    		model.addAttribute("redireccio", "/menuAdministrador/mostrarMenu");
    		model.addAttribute("missatgeInfo", missatgeInfo);
        }

		return "info";
    }


	@PostMapping("/modificarUsuari")
    public String modificarUsuari(Model model, @ModelAttribute("login") String login) {

		LOGGER.info("modificarUsuari");
		LOGGER.debug("login: " + login);
		
		Map<String, Object> resultatMap = serveiUsuari.cercarUsuari(login);

		String resultat = (String) resultatMap.get("RESULTAT");
		
        //En cas d'error, mostrem a l'usuari el missatge informatiu de l'error.
        if (resultat.equalsIgnoreCase("NOK")) {
        	
        	String missatgeError = (String) resultatMap.get("MISSATGE");
            LOGGER.error(missatgeError);
    		model.addAttribute("missatgeError", missatgeError);        
            
            return "error";
        }
        
        //Si la cerca d'usuari s correcta, llavors enviar les dades del usuari a la vista corresponent. 
        else if (resultat.equalsIgnoreCase("OK")) {

        	//Posar en el model els objectes usuari i empleat
        	Usuari usuari = (Usuari) resultatMap.get("USUARI");
        	Empleat empleat = (Empleat) resultatMap.get("EMPLEAT");
 
    		model.addAttribute("usuari", usuari);   	
    		model.addAttribute("empleat", empleat);   		
        }

        return "modificacioUsuari";
    }		

	
	@PostMapping("/guardarModificacioUsuari")
    public String guardarModificacioUsuari(HttpSession session, Model model, @ModelAttribute("idUsuari") Integer idUsuari, @ModelAttribute("login") String login, 
    		@ModelAttribute("contrasenya") String contrasenya, @ModelAttribute("eMail") String eMail) {

		LOGGER.info("guardarModificacioUsuari");
		
		LOGGER.debug("idUsuari: " + idUsuari);	
		LOGGER.debug("login: " + login);
		
        //Recuperar el usuari de dins de la sessi.
		String usuariAdministrador = (String) session.getAttribute("usuari");

		Map<String, String> resultatMap = serveiUsuari.modificaUsuari(usuariAdministrador, idUsuari, login, contrasenya, eMail);
    	
		String resultat = (String) resultatMap.get("RESULTAT");		

        //En cas d'error, mostrem a l'usuari el missatge informatiu de l'error.
        if (resultat.equalsIgnoreCase("NOK")) {
        	
        	String missatgeError = (String) resultatMap.get("MISSATGE");
            LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }

        //Si la modificaci d'usuari s correcta s'informa a l'usuari d'aquest fet.        
        else if (resultat.equalsIgnoreCase("OK")) {
        	
            String missatgeInfo = (String) resultatMap.get("MISSATGE");
            LOGGER.info("INFO: "+ missatgeInfo);
            
    		model.addAttribute("redireccio", "/menuAdministrador/mostrarMenu");
    		model.addAttribute("missatgeInfo", missatgeInfo);
        }
		
		return "info";
    }		


	@PostMapping("/canviarEstatUsuari")
    public String canviarEstatUsuari(Model model, @ModelAttribute("login") String login) {

		LOGGER.info("canviarEstatUsuari");

		LOGGER.debug("login: " + login);
		
		Map<String, Object> resultatMap = serveiUsuari.cercarUsuari(login);

		String resultat = (String) resultatMap.get("RESULTAT");
		
        //En cas d'error, mostrem a l'usuari el missatge informatiu de l'error.
        if (resultat.equalsIgnoreCase("NOK")) {
        	
            String missatgeError = (String) resultatMap.get("MISSATGE");
            LOGGER.error(missatgeError);
    		model.addAttribute("missatgeError", missatgeError);        
            
            return "error";
        }
        
        //Si la cerca de usuari es correcta, llavors enviar les dades del usuari a la vista corresponent. 
        else if (resultat.equalsIgnoreCase("OK")) {

        	//Posar en el model els objectes usuari i empleat
        	Usuari usuari = (Usuari) resultatMap.get("USUARI");
        	Empleat empleat = (Empleat) resultatMap.get("EMPLEAT");

        	Map<String, String> opcionsEstatUsuari = new LinkedHashMap<String, String>();
            opcionsEstatUsuari.put("A", "Actiu");
            opcionsEstatUsuari.put("B", "Baixa");
       
    		model.addAttribute("opcionsEstatUsuari", opcionsEstatUsuari);        
    		model.addAttribute("usuari", usuari);   	
    		model.addAttribute("empleat", empleat);   		
        }
        
		return "canviEstatUsuari";
    }		


	@PostMapping("/guardarEstatUsuari")
    public String guardarEstatUsuari(HttpSession session, Model model, @ModelAttribute("estatUsuariSel") String estatUsuariSel, @ModelAttribute("estatUsuari") String estatUsuari, @ModelAttribute("login") String login) {

		LOGGER.info("guardarEstatUsuari"); 
		
		LOGGER.debug("estatUsuariSel: " +estatUsuariSel);
		LOGGER.debug("estatUsuari: " +estatUsuari);		
		
        //Recuperar el usuari de dins de la sessi.
		String usuariAdministrador = (String) session.getAttribute("usuari");

        try {	

        	serveiUsuari.actualitzaEstatUsuari(usuariAdministrador, login, estatUsuariSel);

        } catch (ExcepcioUsuari eu) {

        	String missatgeError = "Error general de bbdd de tipus: " + eu.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }			
		
        String missatgeInfo = "Canvi de estat del usuari guardat amb xit";
		model.addAttribute("redireccio", "/menuAdministrador/mostrarMenu");
		model.addAttribute("missatgeInfo", missatgeInfo);
		
		return "info";
    }		


	@PostMapping("/consultarUsuari")
    public String consultarUsuari(Model model, @ModelAttribute("login") String login) {

		LOGGER.info("consultarUsuari");
		
		Map<String, Object> resultatMap = serveiUsuari.cercarUsuari(login);
		
		String resultat = (String) resultatMap.get("RESULTAT");
		
        //En cas d'error, mostrem a l'usuari el missatge informatiu de l'error.
        if (resultat.equalsIgnoreCase("NOK")) {
        	
        	String missatgeError = (String) resultatMap.get("MISSATGE");
            LOGGER.error(missatgeError);
    		model.addAttribute("missatgeError", missatgeError);        
            
            return "error";
        }
        
        //Si la cerca de usuari es correcta, llavors enviar les dades del usuari a la vista corresponent. 
        else if (resultat.equalsIgnoreCase("OK")) {

        	//Posar en el model els objectes usuari i empleat
        	Usuari usuari = (Usuari) resultatMap.get("USUARI");
        	Empleat empleat = (Empleat) resultatMap.get("EMPLEAT");
        	
        	String estatEnSistema = utils.formatEstatEnSistema(usuari.getEstatEnSistema());
        	usuari.setEstatEnSistema(estatEnSistema);
        	String dataAlta = utils.formatDate(usuari.getDataAlta());
        	String dataModificacio = utils.formatDate(usuari.getDataModificacio());
        	String dataBaixa = utils.formatDate(usuari.getDataBaixa());
        	
			LOGGER.debug("cercarUsuari: empleat cercat: " + empleat.toString());
			LOGGER.debug("cercarUsuari: usuari cercat: " + usuari.toString());         	
        	
			model.addAttribute("dataAlta", dataAlta);
			model.addAttribute("dataModificacio", dataModificacio);
			model.addAttribute("dataBaixa", dataBaixa);			
    		model.addAttribute("usuari", usuari);        	
    		model.addAttribute("empleat", empleat);    		
      }
		
		return "consultaUsuari";
    }		

}
