package edu.uoc.grp.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.util.List;
import edu.uoc.grp.exceptions.ExcepcioUsuari;
import edu.uoc.grp.service.ServeiUsuari;
import edu.uoc.grp.model.Usuari;
import edu.uoc.grp.model.Empleat;
import edu.uoc.grp.utilities.Utils;


/**
 * Classe ControladorMenuAdministrador.
 * 
 * Classe de tipus controlador encarregada de gestionar les peticions i les respostes relacionades amb les opcions de men del administrador. 
 *
 * @version 1.0; java version "1.8.0";
 */
@Controller
@RequestMapping("/menuAdministrador") 
public class ControladorMenuAdministrador {
	
	private static final Logger LOGGER = LogManager.getLogger(ControladorMenuAdministrador.class.getName());
    
    @Autowired
    private ServeiUsuari serveiUsuari;	

    @Autowired
    private Utils utils;
    

	@GetMapping("/mostrarMenu")
    public String mostrarMenu(Model model) {

		LOGGER.info("mostrarMenu");
		
		return "menuAdministrador";
    }	


	@GetMapping("/seleccioEmpleat")
    public String seleccioEmpleat(Model model) {

		LOGGER.info("seleccioEmpleat");
		List<Empleat> llistatEmpleat;
		
        try {		
    		
        	llistatEmpleat = serveiUsuari.obtenirLlistatEmpleats();

        } catch (ExcepcioUsuari eu) {
        	
            String missatgeError = utils.getMsg("msg.error.general.bd") + eu.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }		

		model.addAttribute("llistatEmpleat", llistatEmpleat);		
		
		return "seleccioEmpleat";
    }


	@GetMapping("/modificacioUsuari")
    public String modificacioUsuari(Model model) {

		LOGGER.info("modificacioUsuari");
		List<Usuari> usuarisList;
		
        try {		
		
        	usuarisList = serveiUsuari.obtenirLlistatUsuaris();
		
        } catch (ExcepcioUsuari eu) {

        	String missatgeError = utils.getMsg("msg.error.general.bd") + eu.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }		

		model.addAttribute("usuarisList", usuarisList);
		model.addAttribute("accio", "modificar");		
		model.addAttribute("redireccio", "/administrador/modificarUsuari");
		
		return "seleccioUsuari";
    }	


	@GetMapping("/canviEstatUsuari")
    public String canviEstatUsuari(Model model) {

		LOGGER.info("canviEstatUsuari");
		List<Usuari> usuarisList;
		
        try {		
		
        	usuarisList = serveiUsuari.obtenirLlistatUsuaris();
		
        } catch (ExcepcioUsuari eu) {

        	String missatgeError = utils.getMsg("msg.error.general.bd") + eu.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }		

		model.addAttribute("usuarisList", usuarisList);
		model.addAttribute("accio", "canviar_estat");		
		model.addAttribute("redireccio", "/administrador/canviarEstatUsuari");
		
		return "seleccioUsuari";
    }


	@GetMapping("/consultaUsuari")
    public String consultaUsuari(Model model) {

		LOGGER.info("consultaUsuari");
		List<Usuari> usuarisList;
		
        try {		
		
        	usuarisList = serveiUsuari.obtenirLlistatUsuaris();
		
        } catch (ExcepcioUsuari eu) {

        	String missatgeError = utils.getMsg("msg.error.general.bd") + eu.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }		

		model.addAttribute("usuarisList", usuarisList);
		model.addAttribute("accio", "consultar");		
		model.addAttribute("redireccio", "/administrador/consultarUsuari");
		
		return "seleccioUsuari";
    }		
	
	
	@GetMapping("/visualitzacioUsuaris")
    public String visualitzacioUsuaris(Model model) {

		LOGGER.info("visualitzacioUsuaris");
		List<Usuari> usuarisList;
		
        try {		
		
        	usuarisList = serveiUsuari.obtenirLlistatUsuaris();
		
        } catch (ExcepcioUsuari eu) {

        	String missatgeError = utils.getMsg("msg.error.general.bd") + eu.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }		

		model.addAttribute("usuarisList", usuarisList);

		return "visualitzacioUsuaris";
    }		


}
