package edu.uoc.grp.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.util.List;
import edu.uoc.grp.exceptions.ExcepcioPlanificacio;
import edu.uoc.grp.service.ServeiPlanificacio;
import edu.uoc.grp.model.Vaixell;
import edu.uoc.grp.utilities.Utils;


/**
 * Classe ControladorMenuPlanificador.
 * 
 * Classe de tipus controlador encarregada de gestionar les peticions i les respostes relacionades amb les opcions de men del planificador. 
 *
 * @version 1.0; java version "1.8.0";
 */
@Controller
@RequestMapping("/menuPlanificador") 
public class ControladorMenuPlanificador {

	private static final Logger LOGGER = LogManager.getLogger(ControladorMenuPlanificador.class.getName());

    @Autowired
    private ServeiPlanificacio serveiPlanificacio;		
    
    @Autowired
    private Utils utils;
    
    
	@GetMapping("/mostrarMenu")
    public String mostrarMenu(Model model) {

		LOGGER.info("mostrarMenu");
		
		return "menuPlanificador";
    }	
	
	
	@GetMapping("/consultaVaixell")
    public String consultaVaixell(Model model) {

		LOGGER.info("consultaVaixell");
		List<Vaixell> vaixellsList;
		
        try {		

        	vaixellsList = serveiPlanificacio.obtenirLlistatVaixells();
		
        } catch (ExcepcioPlanificacio ep) {

            String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }		
						
		model.addAttribute("vaixellsList", vaixellsList); 
		model.addAttribute("accio", "consultarVaixell");		
		model.addAttribute("redireccio", "/planificador/consultarVaixell");
		
		return "seleccioVaixell";
    }	
	
	
	@GetMapping("/visualitzacioVaixells")
    public String visualitzacioVaixells(Model model) {

		LOGGER.info("visualitzacioVaixells");
		List<Vaixell> vaixellsList;
		
        try {		

        	vaixellsList = serveiPlanificacio.obtenirLlistatVaixells();
		
        } catch (ExcepcioPlanificacio ep) {

            String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }		
						
		model.addAttribute("vaixellsList", vaixellsList); 
		
		return "visualitzacioVaixells";
    }
	

	@GetMapping("/gestioAssigAreaVaixell")
    public String gestioAssigAreaVaixell(Model model) {

		LOGGER.info("gestioAssigAreaVaixell");
		List<Vaixell> vaixellsList;
		
        try {		

        	vaixellsList = serveiPlanificacio.obtenirLlistatVaixells();
		
        } catch (ExcepcioPlanificacio ep) {

            String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }		
						
		model.addAttribute("vaixellsList", vaixellsList); 
		model.addAttribute("accio", "assignarArea");		
		model.addAttribute("redireccio", "/planificador/gestionarAssigAreaVaixell");
		
		return "seleccioVaixell";
    }	
	
	
	@GetMapping("/gestioAssigMollVaixell")
    public String gestioAssigMollVaixell(Model model) {

		LOGGER.info("gestioAssigMollVaixell");
		List<Vaixell> vaixellsList;
		
        try {		

        	vaixellsList = serveiPlanificacio.obtenirLlistatVaixells();
		
        } catch (ExcepcioPlanificacio ep) {

            String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }		
						
		model.addAttribute("vaixellsList", vaixellsList); 
		model.addAttribute("accio", "assignarMoll");		
		model.addAttribute("redireccio", "/planificador/gestionarAssigMollVaixell");
		
		return "seleccioVaixell";
    }

	
	@GetMapping("/assigTripVaixell")
    public String assigTripVaixell(Model model) {

		LOGGER.info("assigTripVaixell");
		List<Vaixell> vaixellsList;
		
        try {		

        	vaixellsList = serveiPlanificacio.obtenirLlistatVaixells();
		
        } catch (ExcepcioPlanificacio ep) {

            String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }		
						
		model.addAttribute("vaixellsList", vaixellsList); 
		model.addAttribute("accio", "assignarTrip");		
		model.addAttribute("redireccio", "/planificador/assignarTripVaixell");
		
		return "seleccioVaixell";
    }

	
	@GetMapping("/canviContrasenya")
    public String canviContrasenya(Model model) {

		LOGGER.info("canviContrasenya");
		
		model.addAttribute("tipusUsuari", "planificador");		
		model.addAttribute("urlOrigen", "/GRP2018/menuPlanificador/mostrarMenu");
		
		return "canviContrasenya";
    }

	
}
