package edu.uoc.grp.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.util.LinkedHashMap;
import java.util.Map;
import javax.servlet.http.HttpSession;
import edu.uoc.grp.exceptions.ExcepcioInformacio;
import edu.uoc.grp.service.ServeiInformacio;
import edu.uoc.grp.model.Tripulant;
import edu.uoc.grp.model.Vaixell;
import edu.uoc.grp.model.Area;
import edu.uoc.grp.model.Moll;
import edu.uoc.grp.utilities.Utils;


/**
 * Classe ControladorTripulant.
 * 
 * Classe de tipus controlador encarregada de gestionar les peticions i les respostes relacionades amb les funcionalitats del tripulant.
 *
 * @version 1.0; java version "1.8.0";
 */
@Controller
@RequestMapping("/tripulant") 
public class ControladorTripulant {

	private static final Logger LOGGER = LogManager.getLogger(ControladorTripulant.class.getName());

    @Autowired
    private ServeiInformacio serveiInformacio;		

    @Autowired
    private Utils utils;
    

	@GetMapping("/visualitzacioInfoTripulant")
    public String visualitzacioInfoTripulant(HttpSession session, Model model) {

		LOGGER.info("visualitzacioInfoTripulant");
		Tripulant tripulant;
		Vaixell vaixell;
		Area area;
		Moll moll;
		
        //Recuperar l'identificaci de l'usuari de dins de la sessi.
		String login = (String) session.getAttribute("usuari");
		
		LOGGER.debug("Login: " + login);		
		
		try {
            //Cercar la informaci del tripulant utilitzant el seu l'identificaci de usuari.  
			tripulant = serveiInformacio.cercarTripulant(login);
		
			LOGGER.debug("VaixellAssignat: " + tripulant.getVaixellAssignat());			

			
			//Si el tripulant no t un vaixell assignat, llavors es mostrar un missatge informant d'aquest fet.		
			if(tripulant.getVaixellAssignat() == -1) {
	        	
	            model.addAttribute("redireccio", "/menuTripulant/mostrarMenu");			
	            model.addAttribute("missatgeInfo", utils.getMsg("msg.info.trip.no.assignat"));
	            
				return "info";
			}			

			//Si el tripulant t un vaixell assignat, llavors s'obt la informaci relativa als recursos assignats al vaixell.
			vaixell = serveiInformacio.cercarVaixell(tripulant.getVaixellAssignat());
			
			area = serveiInformacio.cercarArea(vaixell.getIdAreaAssignada());
			moll = serveiInformacio.cercarMoll(vaixell.getIdMollAssignat()); 


        } catch (ExcepcioInformacio ei) {
        	
        	String missatgeError = utils.getMsg("msg.error.general.bd") + ei.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }

		model.addAttribute("area", area);
		model.addAttribute("moll", moll);		
		model.addAttribute("vaixell", vaixell);
		model.addAttribute("tripulant", tripulant);
		model.addAttribute("unitatsExtensio", " Km\u00B2");
		
		return "visualitzacioInfoTripulant";
    }

	
	@GetMapping("/visualitzacioInfoVaixell")
    public String visualitzacioInfoVaixell(HttpSession session, Model model) {

		LOGGER.info("visualitzacioInfoVaixell");
		Map<String, String> tripulacioMap = new LinkedHashMap<String, String>();		
		Vaixell vaixell;
		Area area;
		Moll moll;
		
        //Recuperar l'identificaci de l'usuari de dins de la sessi.
		String login = (String) session.getAttribute("usuari");
		
		LOGGER.debug("Login: " + login);
		
		try {
            //Cercar la informaci del tripulant utilitzant el seu l'identificaci de usuari.  
			Tripulant tripulant = serveiInformacio.cercarTripulant(login);
		
			LOGGER.debug("VaixellAssignat: " + tripulant.getVaixellAssignat());			

			
			//Si el tripulant no t un vaixell assignat, llavors es mostrar un missatge informant d'aquest fet.		
			if(tripulant.getVaixellAssignat() == -1) {

	            model.addAttribute("redireccio", "/menuTripulant/mostrarMenu");			
	            model.addAttribute("missatgeInfo", utils.getMsg("msg.info.trip.no.assignat"));
	            
				return "info";
			}

			//Si el tripulant t un vaixell assignat, llavors s'obt la informaci relativa als recursos assignats al vaixell.
			vaixell = serveiInformacio.cercarVaixell(tripulant.getVaixellAssignat());
			
			area = serveiInformacio.cercarArea(vaixell.getIdAreaAssignada());
			moll = serveiInformacio.cercarMoll(vaixell.getIdMollAssignat());
			
			tripulacioMap = serveiInformacio.obtenirTripulacioVaixell(vaixell.getIdVaixell());


        } catch (ExcepcioInformacio ei) {
        	
        	String missatgeError = utils.getMsg("msg.error.general.bd") + ei.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }

		model.addAttribute("area", area);
		model.addAttribute("moll", moll);		
		model.addAttribute("vaixell", vaixell);
		model.addAttribute("unitatsExtensio", " Km\u00B2");
		model.addAttribute("tripulacioMap", tripulacioMap);		
		
		return "visualitzacioInfoVaixell";
    }	
	
}
