package edu.uoc.grp.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.util.HashMap;
import java.util.Map;
import edu.uoc.grp.exceptions.ExcepcioUsuari;
import edu.uoc.grp.dao.DaoUsuari;
import edu.uoc.grp.model.Usuari;
import edu.uoc.grp.utilities.Utils;


/**
 * Classe ServeiAcces.
 * 
 * Classe de tipus servei que proporciona els mtodes necessaris per a gestionar l'accs a l'aplicaci. 
 *
 * @version 1.0; java version "1.8.0";
 */
@Service("ServeiAcces")
public class ServeiAcces {

	private static final Logger LOGGER = LogManager.getLogger(ServeiAcces.class.getName());	
	
	@Autowired
	private DaoUsuari daoUsuari;

    @Autowired
    private Utils utils;
	

    /**
     * Mtode que serveix per a cercar un usuari a partir del seu nom identificatiu d'usuari, i comprovar si l'usuari
     * s actiu dins del sistema, i si la seva contrasenya s correcta per tal de donar-li accs al sistema.
     *
     * @param login objecte que cont el nom identificatiu del usuari.
     * @param contrasenya objecte que cont la contrasenya del usuari.
     * @return Map<String, String> retorna un mapa amb el resultat de l'operaci d'accs del usuari.
     */	
	public Map<String, String> cercarUsuariAcces(String login, String contrasenya) {
		
		Map<String, String> resultat = new HashMap<String, String>();
        String missatge = " ";
        resultat.put("RESULTAT", " ");
        resultat.put("MISSATGE", missatge);
        Usuari usuari = null;
        Boolean existeixUsuari = false;
        
        try {
    		
        	if(login != null) login = login.trim();
        	
        	existeixUsuari = daoUsuari.usuariExisteix(login);
        	
        } catch (ExcepcioUsuari eu) {
            missatge = utils.getMsg("msg.error.general.bd") + eu.getMessage();  
            resultat.put("RESULTAT", "NOK");
            resultat.put("MISSATGE", missatge);
            return resultat;
        }
        
        //L'usuari no existeix dins de la base de dades
        if (!existeixUsuari) {
            missatge = utils.getMsg("msg.error.usuari.no.trobat.bd");
            resultat.put("RESULTAT", "NOK");
            resultat.put("MISSATGE", missatge);

        //L'usuari existeix dins de la base de dades
        } else {

            try {
            	
            	usuari = daoUsuari.cercarUsuari(login);

            } catch (ExcepcioUsuari eu) {
                missatge = utils.getMsg("msg.error.general.bd") + eu.getMessage(); 
                resultat.put("RESULTAT", "NOK");
                resultat.put("MISSATGE", missatge);
                return resultat;
            }
        	
            if(usuari == null) {
                missatge = utils.getMsg("msg.error.general") + "(cercarUsuariAcces) usuari es null"; 
                resultat.put("RESULTAT", "NOK");
                resultat.put("MISSATGE", missatge);
                return resultat;
            }
            
        	LOGGER.debug("Usuari cercat: " + usuari.toString());
        	
            //Comprovem si el usuari es troba inactiu
            if ("B".equalsIgnoreCase(usuari.getEstatEnSistema())) {
                missatge = utils.getMsg("msg.error.acces.usuari.inactiu");
                resultat.put("RESULTAT", "NOK");
                resultat.put("MISSATGE", missatge);
            }
            //Si el usuari es troba actiu
            else if ("A".equalsIgnoreCase(usuari.getEstatEnSistema())) {

                //Comprovem si la seva contrasenya s incorrecta.
                if (!usuari.getContrasenya().equals(contrasenya)) {
                    missatge = utils.getMsg("msg.error.acces.contrasenya.nok");
                    resultat.put("RESULTAT", "NOK");
                    resultat.put("MISSATGE", missatge);

                //Si la contrasenya s correcta.
                } else {
                    resultat.put("RESULTAT", "OK");
                    resultat.put("TIPUS_USUARI", usuari.getTipusUsuari());
                }
            }
        }

        return resultat;
	}	

	
}
