package edu.uoc.grp.utilities;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.ServletException;
import javax.servlet.FilterChain;
import javax.servlet.Filter;
import java.io.IOException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


/**
 * Classe FiltreSeguretat.
 * 
 * Classe de tipus filtre encarregada de controlar si la sessi ha expirat, i si l'usuari t permisos per accedir a l'apartat que hagi sollicitat. 
 *
 * @version 1.0; java version "1.8.0";
 */
public class FiltreSeguretat implements Filter {

	private static final Logger LOGGER = LogManager.getLogger(FiltreSeguretat.class.getName());	
	
	
	@Override
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
		
		Boolean doFilter = true;
		String servletPath = getServletPath(request);		
		
		//ACCIONS DEL ADMINISTRADOR.
		if(servletPath.toUpperCase().indexOf("ADMINISTRADOR") != -1) {
			 
			//Comprovar si la sessi ha expirat.
			if(esSessioExpirada(request)) {
				
				if(response != null && response instanceof HttpServletResponse) {
					doFilter = false;
					HttpServletResponse res = (HttpServletResponse) response;
					res.sendRedirect("/GRP2018/acces/tempsSessioExpirat");			
				}
			}
			//Comprovar si l'usuari t permisos per accedir a l'acci sollicitada.
			else {
				 
				if(esAccesDenegat(request, "ADMINISTRADOR")) {
					if(response != null && response instanceof HttpServletResponse) {
						doFilter = false;
						HttpServletResponse res = (HttpServletResponse) response;
						res.sendRedirect("/GRP2018/acces/accesDenegat");									 
					}
				}
			}

		//ACCIONS DEL PLANIFICADOR.
		} else if(servletPath.toUpperCase().indexOf("PLANIFICADOR") != -1) { 
			 
			//Comprovar si la sessi ha expirat.
			if(esSessioExpirada(request)) {
				 
				if(response != null && response instanceof HttpServletResponse) {
					doFilter = false;
					HttpServletResponse res = (HttpServletResponse) response;
					res.sendRedirect("/GRP2018/acces/tempsSessioExpirat");									 
				}
			}
			//Comprovar si l'usuari t permisos per accedir a l'acci sollicitada.
			else {
				 
				if(esAccesDenegat(request, "PLANIFICADOR")) {
					if(response != null && response instanceof HttpServletResponse) {
						doFilter = false;
						HttpServletResponse res = (HttpServletResponse) response;
						res.sendRedirect("/GRP2018/acces/accesDenegat");									 
					}
				}
			}

		//ACCIONS DEL TRIPULANT.
		} else if(servletPath.toUpperCase().indexOf("TRIPULANT") != -1) {
			 
			//Comprovar si la sessi ha expirat.
			if(esSessioExpirada(request)) {
				 
				if(response != null && response instanceof HttpServletResponse) {
					doFilter = false;
					HttpServletResponse res = (HttpServletResponse) response;
					res.sendRedirect("/GRP2018/acces/tempsSessioExpirat");									 
				}
			}
			 
			//Comprovar si l'usuari t permisos per accedir a l'acci sollicitada.
			else {
				 
				if(esAccesDenegat(request, "TRIPULANT")) {
					if(response != null && response instanceof HttpServletResponse) {
						doFilter = false;
						HttpServletResponse res = (HttpServletResponse) response;
						res.sendRedirect("/GRP2018/acces/accesDenegat");									 
					}
				}
			}

		}
		
		//Es realitza quan no hi ha una redirecci a la vista de missatge per informar sobre la sessi expirada o l'accs denegat.
		if(doFilter) chain.doFilter(request, response);

	}


    /**
     * Mtode que serveix per a obtenir la direcci o cam (path) de l'acci sollicitada per l'usuari.
     *
     * @param request objecte que cont la petici realitzada des de la part client.
     * @return String retorna un objecte que cont l'acci sollicitada per l'usuari.
     */	
	private String getServletPath(ServletRequest request) {
		
		String servletPath = " ";

		if(request != null && request instanceof HttpServletRequest) {
			
			HttpServletRequest req = (HttpServletRequest) request;
			
			if(req != null) servletPath = req.getServletPath();

			LOGGER.debug("servletPath: " + servletPath);
		}
		
		return servletPath;
	}


    /**
     * Mtode que serveix per a comprovar si la sessi ha expirat.
     *
     * @param request objecte que cont la petici realitzada des de la part client.
     * @return Boolean retorna cert si la sessi ha expirat, altrament retorna fals.
     */	
	private Boolean esSessioExpirada(ServletRequest request) {
		
		Boolean sessioExpirada = false;
		
		if(request != null && request instanceof HttpServletRequest) {
			
			HttpServletRequest req = (HttpServletRequest) request;
			
			if(req != null) {
				LOGGER.debug("RequestedSessionId: " + req.getRequestedSessionId());				 
				LOGGER.debug("RequestedSessionIdValid: " + req.isRequestedSessionIdValid());
				
				if(req.getRequestedSessionId() != null && !req.isRequestedSessionIdValid()) {
					
					LOGGER.debug("S'ha detectat que la sessi ha expirat");
					
					sessioExpirada = true;
				}
			}
		}
		return sessioExpirada;
	}


    /**
     * Mtode que serveix per a comprovar si l'usuari t permisos per accedir a l'apartat que hagi sollicitat.
     *
     * @param request objecte que cont la petici realitzada des de la part client.
     * @param tipusUsuariRequerit objecte que cont el tipus d'usuari que pot accedir. 
     * @return Boolean retorna cert si es denega l'accs, altrament retorna fals.
     */	
	private Boolean esAccesDenegat(ServletRequest request, String tipusUsuariRequerit) {

		Boolean accesDenegat = false;

		if(request != null && request instanceof HttpServletRequest) {
			
			HttpServletRequest req = (HttpServletRequest) request;
			
			if(req != null) {
				
				HttpSession session = req.getSession();
				
				if(session != null) {
					String usuari = (String) session.getAttribute("usuari");
					String tipusUsuari = (String) session.getAttribute("tipusUsuari");
					LOGGER.debug("Session - usuari: " + usuari);
					LOGGER.debug("Session - tipusUsuari: " + tipusUsuari);
					LOGGER.debug("tipusUsuariRequerit: " + tipusUsuariRequerit);			 
					 
					if(usuari == null || tipusUsuari == null || !(tipusUsuari.equalsIgnoreCase(tipusUsuariRequerit))) accesDenegat = true;
				}
			}
		}
		
		LOGGER.debug("accesDenegat s: " + accesDenegat);
		
		return accesDenegat;
	}


}
