package edu.uoc.grp.utilities;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.MessageSource;
import org.springframework.context.NoSuchMessageException;
import org.springframework.stereotype.Component;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;


/**
 * Classe Utils.
 * 
 * Classe de tipus utilitats encarregada de proporcionar aquells mtodes que sn tils per a les gestions en diverses classes de l'aplicaci. 
 *
 * @version 1.0; java version "1.8.0";
 */
@Component
public class Utils {
	
	private static final Logger LOGGER = LogManager.getLogger(Utils.class.getName());
	private static final Locale defaultLocale = new Locale("ca", "ES");

    @Autowired
    private MessageSource messageSource;	

    @Value("${locale.pais}")
    private String pais;
    
    @Value("${locale.idioma}")
    private String idioma;
    

    /**
     * Mtode que serveix per a obternir els textos dels missatges que es troben dins dels fitxers de propietats.
     *
     * @param entry objecte que cont el identificador del missatge a dins del fitxer de propietats.
     * @return String retorna el text del missatge.
     */    
	public String getMsg(String entry) {

		Locale locale = defaultLocale;		
		String message = "";
		
		LOGGER.debug("pais: "+ pais);
		LOGGER.debug("idioma: "+ idioma);
		
		try {
			
			if(pais != null && !pais.isEmpty() && idioma != null && !idioma.isEmpty()) {
				
				locale = new Locale(idioma, pais);
			}
			
		} catch(Exception e) {
			
			LOGGER.error("No s'ha pogut carregar el idioma (locale) del fitxer de propietats (configuration.properties), error de tipus: " + e.getMessage());
			LOGGER.debug("Es carregar el idioma (locale) per defecte");
			locale = defaultLocale;
		}	
		
		try {
			
			message = messageSource.getMessage(entry, null, locale);
		
		} catch(NoSuchMessageException nsme) {
		
			LOGGER.error("No s'ha pogut carregar el missatge del fitxer de propietats (messages_xx_XX.properties), error de tipus: " + nsme.getMessage());
			
		} catch(Exception e) {
			
			LOGGER.error("No s'ha pogut carregar el missatge del fitxer de propietats (messages_xx_XX.properties), error de tipus: " + e.getMessage());			
		}	
		
		return message;
	}
	

    /**
     * Mtode que serveix per a formatar una data per tal de ser mostrada amb el format de dates dia-mes-any.
     *
     * @param data objecte que cont la data que cal formatar.
     * @return dataLiteral retorna la data formatada.
     */
    public String formatDate(Date data) {

        LOGGER.info("formatDate : inici");
        
        if (data == null) return "";

        SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy");
        String dataLiteral = "";

        try {
            dataLiteral = sdf.format(data);
        } catch (Exception e) {
            return "";
        }

        return dataLiteral;
    }	
	
	
    /**
     * Mtode que serveix per a formatar el estat del usuari al sistema.
     *
     * @param estatEnSistema literal que cont el estat del usuari al sistema.
     * @return estatEnSistema retorna el estat del usuari formatat.
     */
    public String formatEstatEnSistema(String estatEnSistema) {
    	
    	if (estatEnSistema.equalsIgnoreCase("A")) {
    		
    		estatEnSistema = "Actiu";
    		
    	} else if (estatEnSistema.equalsIgnoreCase("B")) {

    		estatEnSistema = "Baixa";
    	}
    	
    	return estatEnSistema;
    }	

	
}
