CREATE SCHEMA IF NOT EXISTS `grp2018` ;


USE `grp2018` ;


CREATE TABLE Empleat
(
  idEmpleat INTEGER NOT NULL,
  tipusEmpleat VARCHAR(25) NOT NULL,
  docIdentificatiu VARCHAR(25) UNIQUE NOT NULL,
  nom VARCHAR(25) NOT NULL,
  cognom1 VARCHAR(25) NOT NULL,
  cognom2 VARCHAR(25) NOT NULL,
  adreca VARCHAR(25) NOT NULL,
  ciutat VARCHAR(25) NOT NULL,
  pais VARCHAR(25) NOT NULL,
  telefon VARCHAR(15) NOT NULL,
  dataAlta date, 
  usuariAlta VARCHAR(25), 
  dataModificacio date, 
  usuariModificacio VARCHAR(25), 
  dataBaixa date, 
  usuariBaixa VARCHAR(25),
  CONSTRAINT pk_Empleat PRIMARY KEY (idEmpleat)
);


CREATE TABLE Usuari
(
  idUsuari INTEGER NOT NULL,
  tipusUsuari VARCHAR(25) NOT NULL,
  login VARCHAR(25) UNIQUE NOT NULL,
  contrasenya VARCHAR(25) NOT NULL,
  eMail VARCHAR(25),
  estatEnSistema VARCHAR(1) NOT NULL,
  dataAlta date, 
  usuariAlta VARCHAR(25), 
  dataModificacio date, 
  usuariModificacio VARCHAR(25), 
  dataBaixa date, 
  usuariBaixa VARCHAR(25),
  CONSTRAINT pk_Usuari PRIMARY KEY (idUsuari),
  CONSTRAINT fk_UsuariEmpleat FOREIGN KEY (idUsuari) REFERENCES Empleat(idEmpleat)
);


CREATE TABLE Moll
(
  idMoll INTEGER NOT NULL,
  codiMoll VARCHAR(25) UNIQUE NOT NULL,
  nomPort VARCHAR(25) NOT NULL,
  CONSTRAINT pk_Moll PRIMARY KEY (idMoll)
);


CREATE TABLE Area
(
  idArea INTEGER NOT NULL,
  codiArea VARCHAR(25) UNIQUE NOT NULL,
  extensio INTEGER NOT NULL,
  localitzacioLatitud DECIMAL(9,6) NOT NULL,
  localitzacioLongitud DECIMAL(9,6) NOT NULL,  
  CONSTRAINT pk_Area PRIMARY KEY (idArea)
);


CREATE TABLE Vaixell
(
  idVaixell INTEGER NOT NULL,
  nomVaixell VARCHAR(25) UNIQUE NOT NULL,
  tipusVaixell VARCHAR(25) NOT NULL, 
  eslora VARCHAR(25) NOT NULL,
  CONSTRAINT pk_Vaixell PRIMARY KEY (idVaixell)
);


CREATE TABLE Jornada
(
  idJornada INTEGER AUTO_INCREMENT NOT NULL,
  idVaixell INTEGER UNIQUE,
  idMoll INTEGER UNIQUE,
  idArea INTEGER UNIQUE,
  CONSTRAINT pk_Jornada PRIMARY KEY (idJornada),
  CONSTRAINT fk_JornadaVaixell FOREIGN KEY (idVaixell) REFERENCES Vaixell(idVaixell),  
  CONSTRAINT fk_JornadaMoll FOREIGN KEY (idMoll) REFERENCES Moll(idMoll),
  CONSTRAINT fk_JornadaArea FOREIGN KEY (idArea) REFERENCES Area(idArea)
);


CREATE TABLE Tripulant
(
  idTripulant INTEGER NOT NULL, 
  tipusTripulant VARCHAR(25) NOT NULL,
  idVaixell INTEGER,
  CONSTRAINT pk_Tripulant PRIMARY KEY (idTripulant),
  CONSTRAINT fk_TripulantEmpleat FOREIGN KEY (idTripulant) REFERENCES Empleat(idEmpleat),
  CONSTRAINT fk_TripulantVaixell FOREIGN KEY (idVaixell) REFERENCES Vaixell(idVaixell)
);

